SndBuf buf;
string fname;
// reporting variables
int lenSamples;
dur duration;
float minAmp;
float maxAmp;
float maxDB;
int minSampIndx;
int maxSampIndx;
float p2pAmp;
float absMaxAmp;
float dcOffset;
float avgRMSAmp;
float avgRMSdb;
float maxRMSAmp;
float maxRMSdb;
int maxRMSstartSamp;
int maxRMSendSamp;
float minRMSAmp;
float minRMSdb;
int minRMSstartSamp;
int minRMSendSamp;
float sampRA[0]; // necessary to initialize one element

// init expects the name of an existing wav file
// init must be called before using any other functions
function void init( string filename )
{
	/*
	set the fname class variable to the filename
	read the filename into the class SndBuf object
	create a lenSamples variable and set it to a bad value -1
	set the lenSamples variable to the number of samples in the SndBuf
	set the duration variable to the duration of the SndBuf
	*/
	filename => fname;
	fname => buf.read;
	-1 => lenSamples;
	buf.samples() => lenSamples;
	if ( lenSamples < 1 )
		<<< "Bad sound file length" >>>;
	buf.length() => duration;

	// increase sampRA[] size
	lenSamples => sampRA.size;
	fillSampRA();
	getMinAmp() => minAmp;
	getMaxAmp() => maxAmp;
	getPeak2PeakAmp() => p2pAmp;
	getDCOffset() => dcOffset;
	getAvgRMSAmp() => avgRMSAmp; // also sets avgRMSdb
	getRMSMinAmp( 200 ) => minRMSAmp;  // also sets minRMSdb, minRMSstartSamp, minRMSendSamp
	getRMSMaxAmp( 200 ) => maxRMSAmp;  // also sets maxRMSdb, maxRMSstartSamp, maxRMSendSamp
}

function void info( string header )
{
	/*
	TEST 1 OUTPUT
	==== Test 1 ====  
	Sound file name
	/Users/je/Sites/m208w2014_scrivener/_labs/Lab15/music208minus24dB.wav 
	Length in samples
	108861 
	Duration
	2468.503401 ms
	2.468503 seconds
	0.041142 minutes 
	*/
	<<< "==== " + header + " ====", "" >>>;
	<<< "Sound file name\n\t", fname >>>;
	<<< "Length in samples\n\t", lenSamples >>>;
	<<< "Duration\n\t", duration/ms, "ms\n\t", duration/second, "seconds\n\t", duration/minute, "minutes" >>>;
	// <<< "sampRA[0] to sampRA[4]", sampRA[0], sampRA[1], sampRA[2], sampRA[3] >>>;
	
	<<< "Minimum amplitude", "" >>>;
	<<< "\t", getMinAmp(), "at sample", minSampIndx >>>;
	
	<<< "Maximum amplitude" , "" >>>;
	<<< "\t", getMaxAmp(), "at sample", maxSampIndx >>>;
	
	<<< "Peak to peak amplitude" , "" >>>;
	<<< "\t", getPeak2PeakAmp() >>>;
	
	<<< "DC Offset" , "" >>>;
	<<< "\t", getDCOffset() >>>;
	
	<<< "Average RMS amplitude" , "" >>>;
	<<< "\t", getAvgRMSAmp() >>>;
	<<< "\t", avgRMSdb, "dB" >>>;
	
	<<< "Maximum RMS amplitude", "" >>>;
	<<< "\t", getRMSMaxAmp( 200 ) >>>;
	<<< "\t", maxRMSdb, "dB" >>>;
	<<< "\tsamples", maxRMSstartSamp, "-", maxRMSendSamp  >>>;		
	
	<<< "Minimum RMS amplitude", "" >>>;
	<<< "\t", getRMSMinAmp( 200 ) >>>;
	<<< "\t", minRMSdb, "dB" >>>;
	<<< "\tsamples", minRMSstartSamp, "-", minRMSendSamp  >>>;		
	
	
	<<< "Clipped sample count\n\t", clippedSamples() >>>;
	
}

function void fillSampRA( )
{ 
	// The array variable sampRA[] will hold the SndBuf samples.
	// The sample value at index n in SndBuf  is buf.valueAt(n)
	// The total number of samples in SndBuf is lenSamples
	// use a for loop to set every sampRA[n] to the value of buf.valueAt(n)
	for ( 0 => int n; n < lenSamples; n++ )
	{
		buf.valueAt(n) => sampRA[n];
	}
}

function float db2amp( float dB )
{
	// in Octave code 
	// amp = 10^(dB/20)
	// return amp;
	return Math.pow( 10, dB/20.0 );		
	
}

function float amp2db( float amp )
{
	// in Octave Code
	// dB = 20 * log10( amp );	
	// return dB;
	return 20 * Math.log10( amp );		
}

function float getMinAmp()
{
	// find the minimum sample amplitude and the index where it occurs
	// use a for loop to test every sample value
	// if a sample value is less than minA
	// set minA to that value
	// set minSampIndx variable to that index
	// return minA
	1.0 => float minA;
	for ( 0 => int n; n < lenSamples; n++ )
	{
		if ( sampRA[n] < minA )
		{
			sampRA[n] => minA;
			n => minSampIndx;
		}
	}
	return minA;
}

function float getMaxAmp()
{
	// find the maximum sample amplitude and the index where it occurs
	// use a for loop to test every sample value
	// if a sample value is less than maxA
	// set maxA to that value
	// set maxSampIndx variable to that index
	// return that value
	-1.0 => float maxA;
	for ( 0 => int n; n < lenSamples; n++ )
	{
		if ( sampRA[n] > maxA )
		{
			sampRA[n] => maxA;
			n => maxSampIndx;
			amp2db( maxA ) => maxDB;
		}
	}
	return maxA;
}

function float getPeak2PeakAmp()
{
	// add maxAmp and the absolute value of minAmp
	// to get the peak to peak amplitude
	// return result
	return getMaxAmp() + Math.fabs( getMinAmp() ) ;
}

function float getDCOffset()
{
	// cumulatively add up all the sample values in a for loop
	// divide the sum by the total number of samples
	// return the result - that's the DC offxet
	0.0 => float sum;
	for ( 0 => int n; n < lenSamples; n++ )
	{
		sum + sampRA[n] => sum;	
	}	
	return sum / lenSamples;
}	

function float getAvgRMSAmp()
{
	// use a for loop to add up the sum of all the sample values squared
	// take the square root of the resulting sum, thats rms amplitude
	// convert rms amplitude to dB and set avgRMSdb;
	// return rms
	0.0 => float rms;
	for ( 0 => int n; n < lenSamples; n++ )
	{
		rms + ( sampRA[n] * sampRA[n] ) => rms;
	}
	Math.sqrt( rms / lenSamples ) => rms;
	amp2db( rms ) => avgRMSdb;
	return  rms;		
	
}	

function float getRMSMaxAmp( float millis )
{
	0 => float maxRMS;
	0 => float rms;
	0 => int rmsWinSize;
	0 => int numWins;
	
	// calculate the number of samples in milliseconds
	//    => rmsWinSize;
	// calculate how many "windows" of rmsWinSize can slide
	// across the totnal number of samples in SndBuf
	//    => numWins;
	// OUTER LOOP index w goes from 0 to numWinds
	//    w * rmsWinSize => startN;
	//    0 => rms;
	//    INNER LOOP index n goes from 0 to rmsWinSize
	//    the sample number in the inner loop is
	//        sampRA[ n + startN ]
	//    the inner loop calculates the total sum of the squared amplitudes
	// INSIDE OUTER LOOP
	// check if rms > maxRMS, if it is
	//    rms => maxRMS;
	//    w * rmsWinSize => maxRMSbeginSamp;
	//    maxRMSbeginSamp + rmsWinSize => maxRMSendSamp
	// when both loops finish
	//    square root of (maxRMS / rmsWinSize) => maxRMS
	//    compute amp2db( maxRMS ) and store it in class variable maxRMSdb
	// return maxRMSAmp   
	
	(millis * 44.1) $ int =>  rmsWinSize;
Math.floor( lenSamples / rmsWinSize ) $ int =>  numWins;

for ( 0 => int w; w < numWins; w++ )
{
	w * rmsWinSize => int startN;
	0.0 => rms;
	
	for ( 0 => int n; n < rmsWinSize; n++ )
		rms + ( sampRA[ n + startN ] * sampRA[ n + startN ] ) => rms;
	
	if ( rms > maxRMS )
	{
		rms => maxRMS;
		w * rmsWinSize => maxRMSstartSamp;
		maxRMSstartSamp + rmsWinSize => maxRMSendSamp;
	}
}

Math.sqrt( maxRMS / rmsWinSize ) => maxRMS;
amp2db( maxRMS ) => maxRMSdb;
return maxRMS;
}


function float getRMSMinAmp( float millis )
{
	// similar to getRMSMaxAmp( float millis )
	1.0 => float minRMS;
	0 => float rms;
	0 => int rmsWinSize;
	0 => int numWins;
	
	(millis * 44.1) $ int => rmsWinSize;
	Math.floor( lenSamples / rmsWinSize ) $ int => numWins;
	
	for ( 0 => int w; w < numWins; w++ )
	{
		w * rmsWinSize => int startN;
		0.0 => rms;
		
		for ( 0 => int n; n < rmsWinSize; n++ )
			rms + ( sampRA[ n + startN ] * sampRA[ n + startN ] ) => rms;
		
		if ( (rms > 0) && (rms < minRMS) )
		{
			rms => minRMS;
			w * rmsWinSize => minRMSstartSamp;
			minRMSstartSamp + rmsWinSize => minRMSendSamp;
		}
	}
	Math.sqrt( minRMS / rmsWinSize ) => minRMS;
	amp2db( minRMS ) => minRMSdb;
	return minRMS;
	
}

function void removeDCOffset()
{
	// find the mean by calling getDCOffset()
	// loop through sampRA[] and subract the mean from each sample
	// update the dcOffset class variable
	// getDCOffset() => dcOffset;
	getDCOffset() => dcOffset; // update dcOffset variable
	for ( 0 => int n; n < lenSamples; n++ )
	{
		sampRA[n] - dcOffset  => sampRA[n];	
	}
}

function void normalize()
{
	// call removeDCOffset();
	// get the maximum amplitude
	// get the minimum amplitude
	// find the absolute value of the minimum amplitude
	// choose the larger of maxA or Math.fabs( minA )
	// determine the amplitude scale factor
	//      1/maxA => scaleBy;
	// loop through sampRA and multiply all values by scaleBy
	//	removeDCOffset();
	//	normalizePositive();
	//	normalizeNegative();
	
}

function void amplify( float dB )
{
	// call removeDCOffset()
	// call normalize()
	// determine new scaleBy
	// db2amp( dB ) => float scaleBy
	// loop through sampRA and multiply all values by scaleBy
	removeDCOffset();
	normalize();
	db2amp( dB ) => float scaleBy;
	
	for ( 0 => int n; n < lenSamples; n++ )
	{
		scaleBy * sampRA[n]  => sampRA[n];	
	}			
}

function int clippedSamples()
{
	0 => int count;
	for ( 0 => int n; n < lenSamples; n++ )
	{
		if ( sampRA[n] > 1.0 )
			count++;
		else if (sampRA[n] < -1.0 )
			count++;
	}
	return count;
}

function float[] getSamples()
{
	return sampRA;
}

// TESTING
function void playIt( int play )
{
	if (play != 0) // not equal to zero
	{
		0 => buf.pos;
		buf => dac;
		buf.length() => now;
	}
}

init( me.sourceDir() + "/music208minus24dB.wav" );
// set to 0 to not play, 1 to play
playIt( 0 );
info( "Test 11 before removeDCOffset" );
removeDCOffset();
info( "Test 11 after removeDCOffset" );
/*
<<< "amp2db", amp2db( 0.00001 ) >>>;
<<< "amp2db", amp2db( 0.5 ) >>>;
<<< "amp2db", amp2db( 1.0 ) >>>;
<<< "db2amp", db2amp( 0 ) >>>;
<<< "db2amp", db2amp( -6 ) >>>;
<<< "db2amp", db2amp( -20 ) >>>;
*/
